@@ -37,6 +37,10 @@
 #include <chrono>
 #include <set>
 
+#if OPENDHT_PROXY_SERVER || OPENDHT_PROXY_CLIENT
+#include <json/json.h>
+#endif //OPENDHT_PROXY_SERVER
+
 namespace dht {
 
 struct Value;
@@ -105,6 +109,19 @@ struct OPENDHT_PUBLIC ValueType {
     EditPolicy editPolicy {DEFAULT_EDIT_POLICY};
 };
 
+class TypeStore {
+public:
+    void registerType(const ValueType& type) {
+        types[type.id] = type;
+    }
+    const ValueType& getType(ValueType::Id type_id) const {
+        const auto& t_it = types.find(type_id);
+        return (t_it == types.end()) ? ValueType::USER_DATA : t_it->second;
+    }
+private:
+    std::map<ValueType::Id, ValueType> types {};
+};
+
 /**
  * A "value" is data potentially stored on the Dht, with some metadata.
  *
@@ -170,6 +187,15 @@ struct OPENDHT_PUBLIC Value
                 return f1(v) or f2(v);
             };
         }
+        std::vector<Sp<Value>> filter(const std::vector<Sp<Value>>& values) {
+            if (not (*this))
+                return values;
+            std::vector<Sp<Value>> ret;
+            for (const auto& v : values)
+                if ((*this)(v))
+                    ret.emplace_back(v);
+            return ret;
+        }
     };
 
     /* Sneaky functions disguised in classes */
@@ -346,6 +372,14 @@ struct OPENDHT_PUBLIC Value
     Value(ValueType::Id t, const uint8_t* dat_ptr, size_t dat_len, Id id = INVALID_ID)
      : id(id), type(t), data(dat_ptr, dat_ptr+dat_len) {}
 
+#if OPENDHT_PROXY_SERVER || OPENDHT_PROXY_CLIENT
+    /**
+     * Build a value from a json object
+     * @param json
+     */
+    Value(Json::Value& json);
+#endif //OPENDHT_PROXY_SERVER
+
     template <typename Type>
     Value(ValueType::Id t, const Type& d, Id id = INVALID_ID)
      : id(id), type(t), data(packMsg(d)) {}
@@ -417,6 +451,18 @@ struct OPENDHT_PUBLIC Value
         return ss.str();
     }
 
+#if OPENDHT_PROXY_SERVER || OPENDHT_PROXY_CLIENT
+    /**
+     * Build a json object from a value
+     * Example:
+     * {
+     *  "data":"base64ofdata",
+     *   id":"0", "seq":0,"type":3
+     * }
+     */
+    Json::Value toJson() const;
+#endif //OPENDHT_PROXY_SERVER
+
     /** Return the size in bytes used by this value in memory (minimum). */
     size_t size() const;
 
@@ -424,11 +470,11 @@ struct OPENDHT_PUBLIC Value
     void msgpack_pack_to_sign(Packer& pk) const
     {
         bool has_owner = owner && *owner;
-        pk.pack_map((user_type.empty()?0:1) + (has_owner?(recipient == InfoHash() ? 4 : 5):2));
+        pk.pack_map((user_type.empty()?0:1) + (has_owner?(recipient ? 5 : 4):2));
         if (has_owner) { // isSigned
             pk.pack(std::string("seq"));   pk.pack(seq);
             pk.pack(std::string("owner")); owner->msgpack_pack(pk);
-            if (recipient != InfoHash()) {
+            if (recipient) {
                 pk.pack(std::string("to")); pk.pack(recipient);
             }
         }
@@ -796,6 +842,10 @@ struct OPENDHT_PUBLIC Where
         return ss.str();
     }
 
+    bool empty() const {
+        return filters_.empty();
+    }
+
     OPENDHT_PUBLIC friend std::ostream& operator<<(std::ostream& s, const dht::Where& q);
 
 private:
@@ -946,6 +996,10 @@ unpackVector(const std::vector<std::shared_ptr<Value>>& vals) {
     return ret;
 }
 
+#if OPENDHT_PROXY_SERVER || OPENDHT_PROXY_CLIENT
+uint64_t unpackId(const Json::Value& json, const std::string& key);
+#endif
+
 }
 
 MSGPACK_ADD_ENUM(dht::Value::Field)
